"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = require("debug");
const prefixes = [];
const enabledPrefixes = [];
const errorFuncs = [];
let enabled = false;
function logger(argsPrefix, argsOptions) {
    if (typeof argsPrefix === 'object' && argsPrefix !== null) {
        argsOptions = argsPrefix;
        argsPrefix = 'default';
    }
    let prefix = (argsPrefix && argsPrefix.toString()) || 'default';
    if (/[^\w\d_:-]/ig.test(prefix)) {
        console.error('WARNING: prefix contains unsupported symbols. They are stripped. Only /[^\\w\\d_:-]/ allowed');
        prefix = prefix.replace(/[^\w\d_:-]/ig, '');
    }
    argsOptions = argsOptions || {};
    const options = {
        startWithDebug: !!argsOptions.startWithDebug || false,
        errorsEnabled: !!argsOptions.errorsEnabled || false
    };
    const logFnLog = (0, debug_1.debug)(prefix);
    const logFnError = (0, debug_1.debug)(prefix);
    logFnLog.log = console.log;
    if (options.errorsEnabled) {
        logFnError.enabled = true;
        errorFuncs.push(logFnError);
    }
    if (!prefixes.includes(prefix)) {
        prefixes.push(prefix);
    }
    if (options.startWithDebug) {
        if (!enabledPrefixes.includes(prefix)) {
            enabledPrefixes.push(prefix);
        }
        if (!enabled) {
            debug_1.debug.enable(enabledPrefixes.join(','));
        }
    }
    if (enabled) {
        debug_1.debug.enable(prefixes.join(','));
    }
    const obj = {
        log: logFnLog,
        info: logFnLog,
        warn: logFnLog,
        debug: logFnLog,
        error: logFnError,
        func: (() => { })
    };
    obj.func = (clb) => {
        return logFnLog.enabled && clb(obj);
    };
    return obj;
}
logger.debug = logger;
function listener(signal) {
    const { forceEnable = false, forceDisable = false } = typeof signal === 'object' ? signal : {};
    if (forceEnable && forceDisable) {
        throw new Error('Illegal mix of forceEnable && forceDisable');
    }
    enabled = forceEnable ? true : forceDisable ? false : !enabled;
    console.error(`DEBUG MODE ${enabled ? 'ENABLED' : 'DISABLED'}`);
    if (enabled) {
        debug_1.debug.enable(prefixes.join(','));
    }
    else {
        debug_1.debug.enable(enabledPrefixes.join(','));
        errorFuncs.forEach((logErrFn) => {
            logErrFn.enabled = true;
        });
    }
}
process.on('SIGUSR2', listener);
logger.cleanup = () => {
    process.removeListener('SIGUSR2', listener);
};
exports.default = logger;
